@@ -1,6 +1,6 @@
 /*
  *  methods for encryption/security mechanisms for cryptmount
- *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
  *  (C)Copyright 2005-2008, RW Penney
  */
 
@@ -91,7 +91,7 @@ static struct kmgcry_mode {
 
 static void kmgcry_tx_algnames(const keyinfo_t *keyinfo,
         char **algstr, char **modestr, char **dgststr)
-    /* parse/translate algorithm string into cipher/mode/digest fields */
+    /* Parse/translate algorithm string into cipher/mode/digest fields */
 {   char *buff=NULL, *pos;
     struct map_t {
         const char *src, *dst; }    /* map OpenSSL name to libgcrypt name */
@@ -119,13 +119,13 @@ static void kmgcry_tx_algnames(const keyinfo_t *keyinfo,
     if (keyinfo->cipheralg != NULL && keyinfo->cipheralg[0] != '\0') {
         buff = cm_strdup(keyinfo->cipheralg);
 
-        /* extract cipher-mode from trailing -[^-]* of cipher-name: */
+        /* Extract cipher-mode from trailing -[^-]* of cipher-name: */
         pos = strrchr(buff, '-');
         if (pos != NULL) {
             *modestr = cm_strdup(pos + 1);
             *pos = '\0';
         }
-        /* translate cipher-name to canonical libgcrypt name: */
+        /* Translate cipher-name to canonical libgcrypt name: */
         for (mapent=ctable; mapent->src!=NULL; ++mapent) {
             if (cm_strcasecmp(buff, mapent->src) == 0) {
                 *algstr = cm_strdup(mapent->dst);
@@ -142,7 +142,7 @@ static void kmgcry_tx_algnames(const keyinfo_t *keyinfo,
 
 
     if (keyinfo->digestalg != NULL && keyinfo->digestalg[0] != '\0') {
-        /* translate digest-name to canonical libgcrypt name: */
+        /* Translate digest-name to canonical libgcrypt name: */
         for (mapent=htable; mapent->src!=NULL; ++mapent) {
             if (cm_strcasecmp(mapent->src, keyinfo->digestalg) == 0) {
                 *dgststr = cm_strdup(mapent->dst);
@@ -159,7 +159,7 @@ static void kmgcry_tx_algnames(const keyinfo_t *keyinfo,
 
 static int kmgcry_get_algos(const keyinfo_t *keyinfo,
                     int *cipher, int *ciphermode, int *digest)
-    /* get libgcrypt algorithms for encoding key */
+    /* Get libgcrypt algorithms for encoding key */
 {   char *algstr=NULL, *mdstr=NULL, *dgststr=NULL;
     struct kmgcry_mode *cmd;
     int eflag=ERR_NOERROR;
@@ -181,7 +181,7 @@ static int kmgcry_get_algos(const keyinfo_t *keyinfo,
 
     *digest = gcry_md_map_name(dgststr);
     if (*digest == 0) {
-        fprintf(stderr, _("couldn't find libgcrypt digest \"%s\"\n"), dgststr);
+        fprintf(stderr, _("Couldn't find libgcrypt digest \"%s\"\n"), dgststr);
         eflag = ERR_BADALGORITHM;
         goto bail_out;
     }
@@ -265,13 +265,13 @@ static void kmgcry_keybuilder(gcry_md_hd_t md_hand,
                     const unsigned char *pass, const size_t passlen,
                     unsigned char *ckey, const size_t ckeysz,
                     unsigned char *civ, const size_t civsz)
-    /* generate cipher key & IV from password & salt (default variant) */
+    /* Generate cipher key & IV from password & salt (default variant) */
 {   size_t kpos, ivpos, pos;
     unsigned char *buff;
 
     kpos = ivpos = 0;
     do {
-        /* fold-together password & salt using message-digest: */
+        /* Fold-together password & salt using message-digest: */
         gcry_md_reset(md_hand);
 
         gcry_md_write(md_hand, (const void*)salt, (size_t)kmgcry_saltlen);
@@ -282,7 +282,7 @@ static void kmgcry_keybuilder(gcry_md_hd_t md_hand,
             gcry_md_write(md_hand, (const void*)civ, ivpos); }
         buff = gcry_md_read(md_hand, digest);
 
-        /* transfer message digest into cipher key & initialization vector: */
+        /* Transfer message digest into cipher key & initialization vector: */
         pos = 0;
         while (kpos < ckeysz && pos < mdlen) {
             ckey[kpos++] = buff[pos++]; }
@@ -298,7 +298,7 @@ static void kmgcryossl_keybuilder(gcry_md_hd_t md_hand,
                     const unsigned char *pass, const size_t passlen,
                     unsigned char *ckey, const size_t ckeysz,
                     unsigned char *civ, const size_t civsz)
-    /* generate cipher key & IV from password & salt (a la OpenSSL) */
+    /* Generate cipher key & IV from password & salt (a la OpenSSL) */
 {   size_t kpos, ivpos, pos;
     unsigned char *buff, *prev=NULL;
     unsigned cnt=0;
@@ -307,7 +307,7 @@ static void kmgcryossl_keybuilder(gcry_md_hd_t md_hand,
 
     kpos = ivpos = 0;
     do {
-        /* fold-together password & salt using message-digest: */
+        /* Fold-together password & salt using message-digest: */
         gcry_md_reset(md_hand);
 
         if (cnt > 0) {
@@ -317,14 +317,14 @@ static void kmgcryossl_keybuilder(gcry_md_hd_t md_hand,
         gcry_md_write(md_hand, (const void*)salt, kmgcryossl_saltlen);
         buff = gcry_md_read(md_hand, digest);
 
-        /* transfer message digest into cipher key & initialization vector: */
+        /* Transfer message digest into cipher key & initialization vector: */
         pos = 0;
         while (kpos < ckeysz && pos < mdlen) {
             ckey[kpos++] = buff[pos++]; }
         while (ivpos < civsz && pos < mdlen) {
             civ[ivpos++] = buff[pos++]; }
 
-        /* keep copy of digest to add to next fold: */
+        /* Keep copy of digest to add to next fold: */
         memcpy((void*)prev, (const void*)buff, mdlen);
         ++cnt;
     } while (kpos < ckeysz || ivpos < civsz);
@@ -336,14 +336,14 @@ static void kmgcryossl_keybuilder(gcry_md_hd_t md_hand,
 static int kmgcry_initcipher(int cipher, int ciphermode, int digest,
             const unsigned char *salt, kmgcry_keybuilder_t keybuilder,
             const char *pass, size_t passlen, gcry_cipher_hd_t *hd)
-    /* initialize block cipher from given password, salt & hashing scheme */
+    /* Initialize block cipher from given password, salt & hashing scheme */
 {   gcry_md_hd_t md_hand;
     size_t ckeysz, cblksz, mdlen;
     unsigned char *ckey=NULL, *civ=NULL;
     int eflag=ERR_BADALGORITHM;
 
     if (gcry_cipher_open(hd, cipher, ciphermode, 0) != 0) {
-        fprintf(stderr, "cannot open libgcrypt cipher[%d,%d]\n",
+        fprintf(stderr, "Cannot open libgcrypt cipher[%d,%d]\n",
                 cipher, ciphermode);
         goto bail_out;
     }
@@ -356,7 +356,7 @@ static int kmgcry_initcipher(int cipher, int ciphermode, int digest,
     /* generate cipher key & iv by hashing password: */
     if (keybuilder == NULL) keybuilder = kmgcry_keybuilder;
     if (gcry_md_open(&md_hand, digest, 0) != 0) {
-        fprintf(stderr, "cannot open libgcrypt digest[%d]\n", digest);
+        fprintf(stderr, "Cannot open libgcrypt digest[%d]\n", digest);
         goto bail_out;
     }
     mdlen = gcry_md_get_algo_dlen(digest);
@@ -367,7 +367,7 @@ static int kmgcry_initcipher(int cipher, int ciphermode, int digest,
     /* setup cipher initial state: */
     if (gcry_cipher_setkey(*hd, (void*)ckey, ckeysz) != 0
       || gcry_cipher_setiv(*hd, (void*)civ, cblksz) != 0) {
-        fprintf(stderr, "failed to setup libgcrypt cipher iv[%d,%d]\n",
+        fprintf(stderr, "Failed to setup libgcrypt cipher iv[%d,%d]\n",
                 (int)ckeysz, (int)cblksz);
         goto bail_out;
     }
@@ -384,14 +384,14 @@ static int kmgcry_initcipher(int cipher, int ciphermode, int digest,
 
 static int kmgcry_init_algs()
 {
-    /* nothing needed */
+    /* Nothing needed */
     return 0;
 }
 
 
 static int kmgcry_free_algs()
 {
-    /* nothing needed */
+    /* Nothing needed */
     return 0;
 }
 
@@ -417,7 +417,7 @@ static int kmgcry_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
         return (strcmp(keyinfo->format, "libgcrypt") == 0);
     } else {
         if (fp_key != NULL) {
-            /* check header of existing key-file: */
+            /* check Header of existing key-file: */
             fread((void*)buff, kmgcry_maglen, (size_t)1, fp_key);
             return (strncmp(buff, kmgcry_magstr, kmgcry_maglen) == 0);
         }
@@ -439,7 +439,7 @@ static int kmgcryossl_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
 #endif
     } else {
         if (fp_key != NULL) {
-            /* check header of existing key-file: */
+            /* Check header of existing key-file: */
             fread((void*)buff, kmgcryossl_maglen, (size_t)1, fp_key);
             compat |= (strncmp(buff, kmgcryossl_magstr, kmgcryossl_maglen) == 0);
         }
@@ -449,16 +449,17 @@ static int kmgcryossl_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
 }
 
 
-static int kmgcry_needs_pw(const keyinfo_t *keyinf)
+static unsigned kmgcry_get_properties(const keyinfo_t *keyinf)
 {
-    return 1;
+    return KM_PROP_HASPASSWD;
 }
 
 
 static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
             const km_pw_context_t *pw_ctxt,
-            unsigned char **key, int *keylen, FILE *fp_key)
-    /* extract key from libgcrypt-encrypted file */
+            unsigned char **key, int *keylen, FILE *fp_key,
+	    km_overrides_t *overrides)
+    /* Extract key from libgcrypt-encrypted file */
 {   gcry_cipher_hd_t chd;
     char *passwd=NULL;
     unsigned char *hbuff=NULL, salt[kmgcry_saltlen], *buff=NULL, *bptr;
@@ -476,31 +477,31 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
     eflag = km_get_passwd(ident, pw_ctxt, &passwd, 0, 0);
     if (eflag != ERR_NOERROR) goto bail_out;
 
-    /* read key header: */
+    /* Read key header: */
     fread((void*)hbuff, (size_t)1, kmgcry_maglen, fp_key);
     if (strncmp((const char*)hbuff, kmgcry_magstr, kmgcry_maglen) != 0) {
-        fprintf(stderr, _("bad keyfile format (libgcrypt)\n"));
+        fprintf(stderr, _("Bad keyfile format (libgcrypt)\n"));
         eflag = ERR_BADFILE;
         goto bail_out;
     }
     fread((void*)hbuff, (size_t)1, (size_t)1, fp_key);
     if (hbuff[0] != '\0') {
-        fprintf(stderr, "bad keyfile version [%d]\n", (int)buff[0]);
+        fprintf(stderr, "Bad keyfile version [%d]\n", (int)buff[0]);
         eflag = ERR_BADFILE;
         goto bail_out;
     }
     fread((void*)hbuff, (size_t)4, (size_t)1, fp_key);
     if (hbuff[0] != (cblksz & 0xff) || hbuff[1] != ((cblksz & 0xff00) >> 8)) {
-        fprintf(stderr, "mismatched cipher block size\n");
+        fprintf(stderr, "Mismatched cipher block size\n");
         eflag = ERR_BADFILE;
         goto bail_out;
     }
     *keylen = ((unsigned)hbuff[2]) | (((unsigned)hbuff[3]) << 8);
 
-    /* read salt from keyfile: */
+    /* Read salt from keyfile: */
     fread((void*)salt, (size_t)1, sizeof(salt), fp_key);
 
-    /* read encrypted key from keyfile: */
+    /* Read encrypted key from keyfile: */
     eflag = kmgcry_initcipher(cipher, ciphermode, digest,
                     salt, NULL, passwd, strlen(passwd), &chd);
     if (eflag != ERR_NOERROR) goto bail_out;
@@ -514,11 +515,11 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
     }
     gcry_cipher_close(chd);
 
-    /* verify checksum: */
+    /* Verify checksum: */
     if (!km_aug_verify(buff, (unsigned)*keylen, &chksum0, &chksum)) {
         switch (pw_ctxt->debug_level) {
             case 0:
-                fprintf(stderr, _("password mismatch when extracting key\n"));
+                fprintf(stderr, _("Password mismatch when extracting key\n"));
                 break;
             case 1:     /* fall through... */
             default:
@@ -536,7 +537,7 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
     memcpy(*key, buff, (size_t)*keylen);
 
     if (ferror(fp_key) != 0) {
-        fprintf(stderr, _("key-extraction failed for \"%s\"\n"),
+        fprintf(stderr, _("Key-extraction failed for \"%s\"\n"),
                 keyinfo->filename);
         eflag = ERR_BADFILE;
     }
@@ -553,8 +554,9 @@ static int kmgcry_get_key(const char *ident, const keyinfo_t *keyinfo,
 
 static int kmgcry_put_key(const char *ident, const keyinfo_t *keyinfo,
             const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key)
-    /* store key in libgcrypt-encrypted file */
+            const unsigned char *key, const int keylen, FILE *fp_key,
+	    const km_overrides_t *overrides)
+    /* Store key in libgcrypt-encrypted file */
 {   gcry_cipher_hd_t chd;
     char *passwd=NULL;
     unsigned char hbuff[4], salt[kmgcry_saltlen], *buff=NULL, *bptr;
@@ -568,7 +570,7 @@ static int kmgcry_put_key(const char *ident, const keyinfo_t *keyinfo,
     eflag = km_get_passwd(ident, pw_ctxt, &passwd, 1, 1);
     if (eflag != ERR_NOERROR) goto bail_out;
 
-    /* write key header: */
+    /* Write key header: */
     wrcnt = 0;
     wrcnt += fwrite((const void*)kmgcry_magstr, kmgcry_maglen, (size_t)1, fp_key);
     wrcnt += fwrite((const void*)&kmgcry_version, (size_t)1, (size_t)1, fp_key);
@@ -576,14 +578,14 @@ static int kmgcry_put_key(const char *ident, const keyinfo_t *keyinfo,
     hbuff[2] = (keylen & 0xff); hbuff[3] = (keylen & 0xff00) >> 8;
     wrcnt += fwrite((const void*)hbuff, (size_t)4, (size_t)1, fp_key);
 
-    /* generate salt & record in keyfile: */
+    /* Generate salt & record in keyfile: */
     get_randkey(salt, sizeof(salt));
     wrcnt += fwrite((const void*)salt, sizeof(salt), (size_t)1, fp_key);
 
-    /* augment key with simple checksum: */
+    /* Augment key with simple checksum: */
     buff = km_aug_key(key, (unsigned)keylen, (unsigned)cblksz, &buffsz);
 
-    /* write encrypted key into keyfile: */
+    /* Write encrypted key into keyfile: */
     eflag = kmgcry_initcipher(cipher, ciphermode, digest,
                     salt, NULL, passwd, strlen(passwd), &chd);
     if (eflag != ERR_NOERROR) goto bail_out;
@@ -597,7 +599,7 @@ static int kmgcry_put_key(const char *ident, const keyinfo_t *keyinfo,
     gcry_cipher_close(chd);
 
     if (wrcnt != (4 + buffsz / cblksz)) {
-        fprintf(stderr, _("failed to create new key file\n"));
+        fprintf(stderr, _("Failed to create new key file\n"));
         fprintf(stderr, "wrcnt=%d\n", (int)wrcnt);
         eflag = ERR_BADFILE;
         goto bail_out;
@@ -615,8 +617,9 @@ static int kmgcry_put_key(const char *ident, const keyinfo_t *keyinfo,
 
 static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
             const km_pw_context_t *pw_ctxt,
-            unsigned char **key, int *keylen, FILE *fp_key)
-    /* extract key from OpenSSL-compatible file via libgcrypt */
+            unsigned char **key, int *keylen, FILE *fp_key,
+	    km_overrides_t *overrides)
+    /* Extract key from OpenSSL-compatible file via libgcrypt */
 {   gcry_cipher_hd_t chd;
     char *passwd=NULL;
     unsigned char *hbuff=NULL, salt[kmgcryossl_saltlen], *buff=NULL;
@@ -633,7 +636,7 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
     eflag = km_get_passwd(ident, pw_ctxt, &passwd, 0, 0);
     if (eflag != ERR_NOERROR) goto bail_out;
 
-    /* read key header: */
+    /* Read key header: */
     fread((void*)hbuff, (size_t)1, kmgcryossl_maglen, fp_key);
     if (strncmp((const char*)hbuff, kmgcryossl_magstr, kmgcryossl_maglen) != 0) {
         fprintf(stderr, _("bad keyfile format (openssl-compat)\n"));
@@ -641,7 +644,7 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
         goto bail_out;
     }
 
-    /* read salt from keyfile: */
+    /* Read salt from keyfile: */
     fread((void*)salt, (size_t)1, sizeof(salt), fp_key);
 
     /* read encrypted key from keyfile: */
@@ -660,7 +663,7 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
     }
     gcry_cipher_close(chd);
 
-    /* remove & check end-marker from key-data: */
+    /* Remove & check end-marker from key-data: */
     ofs = 0; idx = 0; kbad = 0;
     if (pos > 0) ofs = buff[pos - 1]; else kbad |= 1;
     if (ofs > cblksz) kbad |= 1;
@@ -671,11 +674,11 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
     if (kbad) {
         switch (pw_ctxt->debug_level) {
             case 0:
-                fprintf(stderr, _("password mismatch when extracting key\n"));
+                fprintf(stderr, _("Password mismatch when extracting key\n"));
                 break;
             case 1:     /* fall through... */
             default:
-                fprintf(stderr, _("checksum mismatch in keyfile (openssl-compat, ofs=%u,idx=%u)\n"),
+                fprintf(stderr, _("Checksum mismatch in keyfile (openssl-compat, ofs=%u,idx=%u)\n"),
                         (unsigned)ofs, (unsigned)idx);
                 break;
         }
@@ -690,7 +693,7 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
     memcpy(*key, buff, (size_t)*keylen);
 
     if (ferror(fp_key) != 0) {
-        fprintf(stderr, _("key-extraction failed for \"%s\"\n"),
+        fprintf(stderr, _("Key-extraction failed for \"%s\"\n"),
                 keyinfo->filename);
         eflag = ERR_BADFILE;
     }
@@ -707,8 +710,9 @@ static int kmgcryossl_get_key(const char *ident, const keyinfo_t *keyinfo,
 
 static int kmgcryossl_put_key(const char *ident, const keyinfo_t *keyinfo,
             const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key)
-    /* store key in OpenSSL-compatible file via libgcrypt */
+            const unsigned char *key, const int keylen, FILE *fp_key,
+	    const km_overrides_t *overrides)
+    /* Store key in OpenSSL-compatible file via libgcrypt */
 {   gcry_cipher_hd_t chd;
     char *passwd=NULL;
     unsigned char salt[kmgcryossl_saltlen], *buff=NULL;
@@ -722,21 +726,21 @@ static int kmgcryossl_put_key(const char *ident, const keyinfo_t *keyinfo,
     eflag = km_get_passwd(ident, pw_ctxt, &passwd, 1, 1);
     if (eflag != ERR_NOERROR) goto bail_out;
 
-    /* write key header: */
+    /* Write key header: */
     wrcnt = 0;
     wrcnt += fwrite((const void*)kmgcryossl_magstr, kmgcryossl_maglen, (size_t)1, fp_key);
 
-    /* generate salt & record in keyfile: */
+    /* Generate salt & record in keyfile: */
     get_randkey(salt, sizeof(salt));
     wrcnt += fwrite((const void*)salt, sizeof(salt), (size_t)1, fp_key);
 
-    /* pad key-data with end-marker: */
+    /* Pad key-data with end-marker: */
     buffsz = cblksz * ((keylen + cblksz) / cblksz);
     buff = sec_realloc(buff, buffsz);
     memcpy((void*)buff, (const void*)key, (size_t)keylen);
     for (pos=keylen; pos<buffsz; ++pos) buff[pos] = (buffsz - keylen);
 
-    /* write encrypted key into keyfile: */
+    /* Write encrypted key into keyfile: */
     eflag = kmgcry_initcipher(cipher, ciphermode, digest,
                     salt, kmgcryossl_keybuilder, passwd, strlen(passwd), &chd);
     if (eflag != ERR_NOERROR) goto bail_out;
@@ -749,7 +753,7 @@ static int kmgcryossl_put_key(const char *ident, const keyinfo_t *keyinfo,
     gcry_cipher_close(chd);
 
     if (wrcnt != (2 + buffsz / cblksz)) {
-        fprintf(stderr, _("failed to create new key file\n"));
+        fprintf(stderr, _("Failed to create new key file\n"));
         fprintf(stderr, "wrcnt=%d\n", (int)wrcnt);
         eflag = ERR_BADFILE;
         goto bail_out;
@@ -852,13 +856,14 @@ static int kmgcry_runtests()
 
 keymanager_t keymgr_gcryossl = {
     "openssl-compat", 0,    kmgcry_init_algs, kmgcry_free_algs,
-                      kmgcry_mk_default, kmgcryossl_is_compat, kmgcry_needs_pw,
+                      kmgcry_mk_default, kmgcryossl_is_compat,
+                      kmgcry_get_properties,
                       kmgcryossl_get_key, kmgcryossl_put_key,
                       kmgcry_md_prepare, kmgcry_md_block,
                       kmgcry_md_final, kmgcry_md_release,
     NULL
 #ifdef TESTING
-    , NULL, NULL
+    , NULL, NULL, 0
 #endif
 };
 
@@ -866,7 +871,8 @@ keymanager_t keymgr_gcryossl = {
 
 keymanager_t keymgr_gcry = {
     "libgcrypt", 0,   kmgcry_init_algs, kmgcry_free_algs,
-                      kmgcry_mk_default, kmgcry_is_compat, kmgcry_needs_pw,
+                      kmgcry_mk_default, kmgcry_is_compat,
+                      kmgcry_get_properties,
                       kmgcry_get_key, kmgcry_put_key,
                       kmgcry_md_prepare, kmgcry_md_block,
                       kmgcry_md_final, kmgcry_md_release,
@@ -876,7 +882,7 @@ keymanager_t keymgr_gcry = {
     NULL
 #endif
 #ifdef TESTING
-    , kmgcry_testctxt, kmgcry_runtests
+    , kmgcry_testctxt, kmgcry_runtests, 0
 #endif
 };
 
