@@ -1,6 +1,6 @@
 /*
  *  methods for encryption/security mechanisms for cryptmount
- *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
  *  (C)Copyright 2005-2008, RW Penney
  */
 
@@ -51,12 +51,12 @@
 #ifndef AS_MODULE
 
 
-/* list of all available key-managers (to be constructed later): */
+/* List of all available key-managers (to be constructed later): */
 static keymanager_t *keymgrs = NULL;
 
 
 /*
- *  ==== raw key-management routines ====
+ *  ==== Raw key-management routines ====
  */
 
 static int kmraw_init_algs(void)
@@ -99,7 +99,7 @@ static int kmraw_is_compat(const keyinfo_t *keyinfo, FILE *fp_key)
 }
 
 
-static int kmraw_needs_pw(const keyinfo_t *keyinf)
+static unsigned kmraw_get_properties(const keyinfo_t *keyinf)
 {
     return 0;
 }
@@ -107,8 +107,9 @@ static int kmraw_needs_pw(const keyinfo_t *keyinf)
 
 static int kmraw_get_key(const char *ident, const keyinfo_t *keyinfo,
             const km_pw_context_t *pw_ctxt,
-            unsigned char **key, int *keylen, FILE *fp_key)
-    /* extract key from unencrypted (plain) file */
+            unsigned char **key, int *keylen, FILE *fp_key,
+            km_overrides_t *overrides)
+    /* Extract key from unencrypted (plain) file */
 {   enum { BUFFSZ=512 };
     char buff[BUFFSZ];
     size_t len, lmt;
@@ -122,23 +123,23 @@ static int kmraw_get_key(const char *ident, const keyinfo_t *keyinfo,
     }
 
 
-    /* read data directly from keyfile: */
+    /* Read data directly from keyfile: */
     for (;;) {
         lmt = (keyinfo->maxlen > 0 && (*keylen + BUFFSZ) > keyinfo->maxlen
                 ? (size_t)(keyinfo->maxlen - *keylen) : (size_t)BUFFSZ);
         len = fread((void*)buff, (size_t)1, (size_t)lmt, fp_key);
         if (len == 0) break;
 
-        /* copy new block of data onto end of current key: */
+        /* Copy new block of data onto end of current key: */
         *key = (unsigned char*)sec_realloc((void*)*key, (size_t)(*keylen+len));
         memcpy((void*)(*key + *keylen), (const void*)buff, len);
         *keylen += len;
     }
 
     if (ferror(fp_key) != 0) {
-        fprintf(stderr, _("key-extraction failed for \"%s\"\n"),
+        fprintf(stderr, _("Key-extraction failed for \"%s\"\n"),
                 keyinfo->filename);
-        /* this is a trivial case of decryption failure: */
+        /* This is a trivial case of decryption failure: */
         eflag = ERR_BADDECRYPT;
     }
 
@@ -150,14 +151,15 @@ static int kmraw_get_key(const char *ident, const keyinfo_t *keyinfo,
 
 static int kmraw_put_key(const char *ident, const keyinfo_t *keyinfo,
             const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key)
-    /* store key in unencrypted (plain) file */
+            const unsigned char *key, const int keylen, FILE *fp_key,
+            const km_overrides_t *overrides)
+    /* Store key in unencrypted (plain) file */
 {   int eflag=ERR_NOERROR;
 
-    /* write data directly into keyfile: */
+    /* Write data directly into keyfile: */
     if (fwrite((const void*)key, (size_t)keylen, (size_t)1, fp_key) == 0
       || ferror(fp_key) != 0) {
-        fprintf(stderr, _("key-writing failed for \"%s\"\n"),
+        fprintf(stderr, _("Key-writing failed for \"%s\"\n"),
                 keyinfo->filename);
         eflag = ERR_BADENCRYPT;
     }
@@ -178,7 +180,7 @@ static void *kmraw_md_prepare(void)
 static void kmraw_md_block(void *state, unsigned char *buff, size_t len)
 {   cm_sha1_ctxt_t *ctxt=(cm_sha1_ctxt_t*)state;
 
-    /* apply built-in SHA1 hash to data-block: */
+    /* Apply built-in SHA1 hash to data-block: */
     cm_sha1_block(ctxt, buff, len);
 }
 
@@ -199,13 +201,14 @@ static void kmraw_md_release(void *state)
 
 static keymanager_t keymgr_raw = {
     "raw", 0,       kmraw_init_algs, kmraw_free_algs,
-                    kmraw_mk_default, kmraw_is_compat, kmraw_needs_pw,
+                    kmraw_mk_default, kmraw_is_compat,
+                    kmraw_get_properties,
                     kmraw_get_key, kmraw_put_key,
                     kmraw_md_prepare, kmraw_md_block,
                     kmraw_md_final, kmraw_md_release,
     NULL
 #ifdef TESTING
-    , NULL, NULL
+    , NULL, NULL, 0
 #endif
 };
 
@@ -248,6 +251,7 @@ static void build_keymgrs(void)
     ptr = &keymgrs;
     ptr = add_keymgr(ptr, kmblti_gethandle());
     ptr = add_keymgr(ptr, kmgcry_gethandle());
+    ptr = add_keymgr(ptr, kmluks_gethandle());
     ptr = add_keymgr(ptr, &keymgr_raw);
 }
 
@@ -272,7 +276,7 @@ const char **get_keymgr_list(void)
 
 
 keymanager_t *get_keymanager(const struct keyinfo *keyinfo, FILE *fp_key)
-    /* search for keymanager that supports given key */
+    /* Search for keymanager that supports given key */
 {   keymanager_t *km;
 
     if (keyinfo == NULL) return NULL;
@@ -318,15 +322,16 @@ int free_keymanagers(void)
 #endif  /* !AS_MODULE */
 
 
-int get_key(const char *ident, const keyinfo_t *keyinfo,
-            const km_pw_context_t *pw_ctxt, unsigned char **key, int *keylen)
+int cm_get_key(const char *ident, const keyinfo_t *keyinfo,
+            const km_pw_context_t *pw_ctxt, unsigned char **key, int *keylen,
+            km_overrides_t *overrides)
 {   keymanager_t *km;
     FILE *fp=NULL;
     unsigned attempts=0;
     int eflag=ERR_NOERROR;
 
     if (keyinfo == NULL || keyinfo->filename == NULL) {
-        fprintf(stderr, _("missing key-file for target \"%s\"\n"), ident);
+        fprintf(stderr, _("Missing key-file for target \"%s\"\n"), ident);
         eflag = ERR_BADFILE;
         goto bail_out;
     }
@@ -334,7 +339,7 @@ int get_key(const char *ident, const keyinfo_t *keyinfo,
     fp = fopen(keyinfo->filename, "rb");
     if (fp == NULL) {
         fprintf(stderr,
-            _("failed to open keyfile \"%s\" for target \"%s\"\n"),
+            _("Failed to open keyfile \"%s\" for target \"%s\"\n"),
             keyinfo->filename, ident);
         eflag = ERR_BADFILE;
         goto bail_out;
@@ -343,12 +348,13 @@ int get_key(const char *ident, const keyinfo_t *keyinfo,
     if ((km = get_keymanager(keyinfo, fp)) != NULL) {
         do {
             (void)fseek(fp, 0L, SEEK_SET);
-            eflag = km->get_key(ident, keyinfo, pw_ctxt, key, keylen, fp);
+            eflag = km->get_key(ident, keyinfo, pw_ctxt, key, keylen,
+                                fp, overrides);
             if (eflag == ERR_BADDECRYPT) sleep(1);
         } while (++attempts < keyinfo->retries && eflag == ERR_BADDECRYPT);
     } else {
         fprintf(stderr,
-                _("unrecognized key format (%s) for target \"%s\"\n"),
+                _("Unrecognized key format (%s) for target \"%s\"\n"),
                 (keyinfo->format != NULL ? keyinfo->format : "NULL"), ident);
         eflag = ERR_BADKEYFORMAT;
     }
@@ -360,22 +366,24 @@ int get_key(const char *ident, const keyinfo_t *keyinfo,
 }
 
 
-int put_key(const char *ident, const keyinfo_t *keyinfo,
+int cm_put_key(const char *ident, const keyinfo_t *keyinfo,
             const km_pw_context_t *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key)
+            const unsigned char *key, const int keylen, FILE *fp_key,
+            const km_overrides_t *overrides)
 {   keymanager_t *km;
     int eflag = ERR_NOERROR;
 
     if (fp_key == NULL) {
-        fprintf(stderr, _("missing output stream for target \"%s\"\n"), ident);
+        fprintf(stderr, _("Missing output stream for target \"%s\"\n"), ident);
         return ERR_BADFILE;
     }
 
     if ((km = get_keymanager(keyinfo, NULL)) != NULL) {
-        eflag = km->put_key(ident, keyinfo, pw_ctxt, key, keylen, fp_key);
+        eflag = km->put_key(ident, keyinfo, pw_ctxt, key, keylen,
+                            fp_key, overrides);
     } else {
         fprintf(stderr,
-                _("unrecognized key format (%s) for target \"%s\"\n"),
+                _("Unrecognized key format (%s) for target \"%s\"\n"),
                 (keyinfo->format != NULL ? keyinfo->format : "NULL"), ident);
         eflag = ERR_BADKEYFORMAT;
     }
@@ -384,15 +392,69 @@ int put_key(const char *ident, const keyinfo_t *keyinfo,
 }
 
 
-int is_keysafe(const keyinfo_t *keyinfo)
-    /* check whether key is stored in encrypted form */
+unsigned cm_get_keyproperties(const keyinfo_t *keyinfo)
+    /* Extract information about key, e.g. whether encrypted */
 {   keymanager_t *km;
+    unsigned props;
 
-    if ((km = get_keymanager(keyinfo, NULL)) == NULL) {
-        return 0;
+    if ((km = get_keymanager(keyinfo, NULL)) != NULL) {
+        props = km->get_properties(keyinfo);
+    } else {
+        /* Safest to assume that key shouldn't be overwritten: */
+        props = KM_PROP_FIXEDLOC | KM_PROP_FORMATTED;
     }
 
-    return km->needs_pw(keyinfo);
+    return props;
+}
+
+
+km_overrides_t *alloc_overrides()
+{   km_overrides_t *overrides;
+
+    overrides = (km_overrides_t*)malloc(sizeof(km_overrides_t));
+
+    overrides->mask = 0;
+    overrides->cipher = NULL;
+    overrides->km_data = NULL;
+
+    return overrides;
+}
+
+void merge_overrides(const cment_t *ent, const km_overrides_t *overrides,
+                    cment_t *merged)
+    /* Combine parameters suggested by key-manager with cment_t */
+{   int64_t delta;
+
+    if (ent == NULL || overrides == NULL) return;
+
+    memcpy((void*)merged, (const void*)ent, sizeof(cment_t));
+
+    if ((overrides->mask & KM_OR_START) != 0) {
+        delta = (overrides->start - ent->start);
+        if (delta >= 0) {
+            merged->start = overrides->start;
+            if (merged->length >= 0) merged->length -= delta;
+        }
+    }
+
+    if ((overrides->mask & KM_OR_CIPHER) != 0) {
+        merged->cipher = overrides->cipher;
+    }
+
+    if ((overrides->mask & KM_OR_IVOFFSET) != 0) {
+        merged->ivoffset = overrides->ivoffset;
+    }
+}
+
+
+void free_overrides(km_overrides_t *overrides)
+{
+    if (overrides == NULL) return;
+
+    if (overrides->cipher != NULL) free((void*)overrides->cipher);
+    if (overrides->km_data != NULL) free((void*)overrides->km_data);
+
+    free((void*)overrides);
 }
 
 
@@ -417,16 +479,19 @@ int km_test_keyrw(void)
 {   enum { MAXKEY=256 };
     keymanager_t *km;
     keyinfo_t keyinfo;
-    int i,keylen,keylen1;
+    int i, keylen=0, keylen1;
     char str[256];
     unsigned char key0[MAXKEY],*key1=NULL;
     FILE *fp;
+    km_overrides_t *overrides=NULL;
     extern cm_testinfo_t *test_ctxtptr;
 
     CM_TEST_START("key read-write");
 
     build_keymgrs();
     for (km=keymgrs; km!=NULL; km=km->next) {
+        if ((km->test_flags & CM_READONLY) != 0) continue;
+
         keyinfo.format = NULL;
         keyinfo.filename = "NOWHERE";
         keyinfo.digestalg = NULL;
@@ -436,26 +501,29 @@ int km_test_keyrw(void)
         km->init_algs();
 
         for (keylen=1; keylen<=MAXKEY; keylen<<=2) {
-            sprintf(str, "key read-write, %s, keylen=%d",
+            sprintf(str, "Key read-write, %s, keylen=%d",
                 km->ident, keylen);
             CM_TEST_IDENT(str);
 
-            /* generate (low-entropy) key: */
+            /* Generate (low-entropy) key: */
             for (i=0; i<keylen; ++i) {
                 key0[i] = (i * 0x9d) ^ ((keylen * (unsigned long)km) % 253);
             }
 
-            /* write key to file: */
+            /* Write key to file: */
             fp = tmpfile();
             if (fp == NULL) CM_TEST_ABORT(context);
-            i = km->put_key("TEST-OUT", &keyinfo, NULL, key0, keylen, fp);
+            overrides = alloc_overrides();
+            i = km->put_key("TEST-OUT", &keyinfo, NULL,
+                            key0, keylen, fp, overrides);
             CM_ASSERT_EQUAL(ERR_NOERROR, i);
 
             key1 = NULL; keylen1 = -keylen;
 
-            /* try reading key back from file: */
+            /* Try reading key back from file: */
             rewind(fp);
-            i = km->get_key("TEST-IN", &keyinfo, NULL, &key1, &keylen1, fp);
+            i = km->get_key("TEST-IN", &keyinfo, NULL,
+                            &key1, &keylen1, fp, overrides);
             CM_ASSERT_EQUAL(ERR_NOERROR, i);
             CM_ASSERT_EQUAL(keylen, keylen1);
             CM_ASSERT_DIFFERENT(key0, key1);
@@ -464,6 +532,7 @@ int km_test_keyrw(void)
                 CM_ASSERT_EQUAL(key0[i], key1[i]);
             }
 
+            free_overrides(overrides);
             fclose(fp);
             sec_free((void*)key1);
         }
@@ -472,7 +541,7 @@ int km_test_keyrw(void)
         free((void*)keyinfo.cipheralg);
         free((void*)keyinfo.digestalg);
     }
-
+    CM_ASSERT_DIFFERENT(keylen, 0);
 
     CM_TEST_OK(context);
 }
@@ -486,7 +555,7 @@ int km_test_keyrw(void)
  */
 
 size_t mk_key_string(const unsigned char *key, const size_t keylen, char *buff)
-    /* create text version of crypto key */
+    /* Create text version of crypto key */
 {   size_t i;
 
     for (i=0; i<keylen; ++i) {
@@ -498,26 +567,26 @@ size_t mk_key_string(const unsigned char *key, const size_t keylen, char *buff)
 
 
 int sycheck_directory(const char *dirname)
-    /* check that permissions on directory are suitably restrictive */
+    /* Check that permissions on directory are suitably restrictive */
 {   struct stat sdir;
 
-    /* get information about directory (if present): */
+    /* Get information about directory (if present): */
     errno = 0;
     if (stat(dirname,&sdir) != 0) {
         if (errno == ENOENT) return ERR_NOERROR;
-        fprintf(stderr, "cannot open \"%s\"\n", dirname);
+        fprintf(stderr, "Cannot open \"%s\"\n", dirname);
         return ERR_INSECURE;
     }
 
-    /* check file/directory ownerships: */
+    /* Check file/directory ownerships: */
     if (sdir.st_uid != (uid_t)0) {
         fprintf(stderr, "\"%s\" must be owned by root\n", dirname);
         return ERR_INSECURE;
     }
 
-    /* check that directory isn't globally writable: */
+    /* Check that directory isn't globally writable: */
     if (!S_ISDIR(sdir.st_mode) || (sdir.st_mode & S_IWOTH) != 0) {
-        fprintf(stderr, "lax permissions on \"%s\"\n", dirname);
+        fprintf(stderr, "Lax permissions on \"%s\"\n", dirname);
         return ERR_INSECURE;
     }
 
@@ -526,12 +595,12 @@ int sycheck_directory(const char *dirname)
 
 
 int sycheck_cmtab(const char *cmtab)
-    /* check that permissions on ${sysconfdir}/cryptmount/cmtab are sensible */
+    /* Check that permissions on ${sysconfdir}/cryptmount/cmtab are sensible */
 {   struct stat sfile;
     char *dirname=NULL;
     int pos, eflag=ERR_NOERROR;
 
-    /* extract directory name from cmtab filename: */
+    /* Extract directory name from cmtab filename: */
     pos = (int)strlen(cmtab);
     dirname = (char*)malloc((size_t)(pos + 1));
     for ( ; pos>0 && cmtab[pos-1] != '/'; --pos) dirname[pos] = '\0';
@@ -540,21 +609,21 @@ int sycheck_cmtab(const char *cmtab)
     if (eflag != ERR_NOERROR) goto bail_out;
 
     if (stat(cmtab,&sfile) != 0) {
-        fprintf(stderr, "cannot open \"%s\"\n", cmtab);
+        fprintf(stderr, "Cannot open \"%s\"\n", cmtab);
         eflag = ERR_INSECURE;
         goto bail_out;
     }
 
-    /* check file ownerships: */
+    /* Check file ownerships: */
     if (sfile.st_uid != (uid_t)0) {
         fprintf(stderr, "\"%s\" must be owned by root\n", cmtab);
         eflag = ERR_INSECURE;
         goto bail_out;
     }
 
-    /* check that file isn't globally writable: */
+    /* Check that file isn't globally writable: */
     if (!S_ISREG(sfile.st_mode) || (sfile.st_mode & S_IWOTH) != 0) {
-        fprintf(stderr, "lax permissions on \"%s\"\n", cmtab);
+        fprintf(stderr, "Lax permissions on \"%s\"\n", cmtab);
         eflag = ERR_INSECURE;
         goto bail_out;
     }
@@ -568,7 +637,7 @@ int sycheck_cmtab(const char *cmtab)
 
 
 static int sy_path(const char *path)
-    /* check whether pathname is considered secure */
+    /* Check whether pathname is considered secure */
 {
     if (path == NULL) return ERR_NOERROR;
     if (path[0] == '/') return ERR_NOERROR;
@@ -577,7 +646,7 @@ static int sy_path(const char *path)
 }
 
 int sycheck_target(const cment_t *cment)
-    /* check that paths within target-specification are sensible */
+    /* Check that paths within target-specification are sensible */
 {   int eflag=ERR_NOERROR;
 
     if (cment == NULL) return 0;
@@ -587,7 +656,7 @@ int sycheck_target(const cment_t *cment)
     eflag |= sy_path(cment->key.filename);
 
     if (eflag != ERR_NOERROR) {
-        fprintf(stderr, "specification for target \"%s\" contains non-absolute pathname\n", cment->ident);
+        fprintf(stderr, "Specification for target \"%s\" contains non-absolute pathname\n", cment->ident);
     }
 
     return eflag;
@@ -600,7 +669,7 @@ int sycheck_target(const cment_t *cment)
  */
 
 int cm_mutex_lock(void)
-    /* try to acquire lock on configuration directory (via symlink marker) */
+    /* Try to acquire lock on configuration directory (via symlink marker) */
 {   char *fname=NULL, ident[64];
     int tries=10, eflag=ERR_BADMUTEX;
 #if HAVE_NANOSLEEP
@@ -615,11 +684,11 @@ int cm_mutex_lock(void)
     while (tries-->0) {
         errno = 0;
         if (symlink(ident, fname) == 0) {
-            /* lock acquired */
+            /* Lock acquired */
             eflag = 0; break;
         } else {
             if (errno == EEXIST) {
-                /* try again later */
+                /* Try again later */
 #if HAVE_NANOSLEEP
                 ticks = (53 * tries + 97 * (long)&tries) % 11;
                 delay.tv_sec = (ticks / 10);
@@ -634,7 +703,7 @@ int cm_mutex_lock(void)
     }
 
     if (eflag != 0) {
-        fprintf(stderr, "failed to create lock-file \"%s\" (errno=%d)\n",
+        fprintf(stderr, "Failed to create lock-file \"%s\" (errno=%d)\n",
                 fname, errno);
     }
 
@@ -644,7 +713,7 @@ int cm_mutex_lock(void)
 }
 
 int cm_mutex_unlock(void)
-    /* release lock on configuration directory */
+    /* Release lock on configuration directory */
 {   char *fname=NULL;
     struct stat sbuff;
     int eflag=0;
@@ -654,7 +723,7 @@ int cm_mutex_unlock(void)
     if (lstat(fname, &sbuff) != 0
       || !S_ISLNK(sbuff.st_mode)
       || unlink(fname) != 0) {
-        fprintf(stderr, "failed to remove lock-file \"%s\"\n", fname);
+        fprintf(stderr, "Failed to remove lock-file \"%s\"\n", fname);
         eflag = ERR_BADMUTEX;
     }
 
@@ -663,6 +732,7 @@ int cm_mutex_unlock(void)
     return eflag;
 }
 
+/* vim: set ts=4 sw=4 et: */
 
 /*
  *  (C)Copyright 2005-2008, RW Penney
