@@ -1,6 +1,6 @@
 /*
  *  declarations for encryption/security mechanisms for cryptmount
- *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
  *  (C)Copyright 2005-2008, RW Penney
  */
 
@@ -30,7 +30,9 @@ struct keyinfo;
 struct cment;
 struct cm_testinfo;
 struct km_pw_context;
+struct km_overrides;
 
+/* Abstract interface to manager of filesystem access keys: */
 typedef struct keymanager {
     const char *ident;
 
@@ -38,40 +40,66 @@ typedef struct keymanager {
     int (*init_algs)(void);
     int (*free_algs)(void);
 
-    /* replace NULL algo-fields in keyinfo with suitable defaults: */
+    /* Replace NULL algo-fields in keyinfo with suitable defaults: */
     void (*mk_default)(struct keyinfo *keyinfo);
 
-    /* determine whether this keymanager can handle given key: */
+    /* Determine whether this keymanager can handle given key: */
     int (*is_compat)(const struct keyinfo *keyinfo, FILE *fp_key);
 
-    /* check if a password is needed to access the key: */
-    int (*needs_pw)(const struct keyinfo *keyinfo);
+    /* Get properties, e.g. whether a password is needed for access: */
+    unsigned (*get_properties)(const struct keyinfo *keyinfo);
 
-    /* extract encrypted key from file: */
+    /* Extract encrypted key from file: */
     int (*get_key)(const char *ident, const struct keyinfo *keyinfo,
                     const struct km_pw_context *pw_ctxt,
-                    unsigned char **key, int *keylen, FILE *fp_key);
+                    unsigned char **key, int *keylen, FILE *fp_key,
+                    struct km_overrides *overrides);
 
-    /* write encrypted key into file: */
+    /* Write encrypted key into file: */
     int (*put_key)(const char *ident, const struct keyinfo *keyinfo,
                     const struct km_pw_context *pw_ctxt,
-                    const unsigned char *key, const int keylen, FILE *fp_key);
+                    const unsigned char *key, const int keylen, FILE *fp_key,
+                    const struct km_overrides *overrides);
 
-    /* message-digest computation (used as source of entropy): */
+    /* Message-digest computation (used as source of entropy): */
     void* (*md_prepare)(void);
     void (*md_block)(void *state, unsigned char *buff, size_t len);
     void (*md_final)(void *state, unsigned char **mdval, size_t *mdlen);
     void (*md_release)(void *state);
 
-    /* linked-list scaffolding: */
+    /* Linked-list scaffolding: */
     struct keymanager *next;
 
 #ifdef TESTING
     void (*install_testctxt)(struct cm_testinfo *context);
     int (*run_tests)(void);
+    unsigned test_flags;
 #endif
 } keymanager_t;
 
+/* Key-manager key-properties flags: */
+enum {
+    KM_PROP_HASPASSWD =     0x001,      /* Password needed to access key */
+    KM_PROP_FIXEDLOC =      0x002,      /* Key-file cannot be renamed */
+    KM_PROP_FORMATTED =     0x004       /* Key-file has been formatted */
+};
+
+
+/* Allow key-manager to override some parameters in struct cment: */
+typedef struct km_overrides {
+    unsigned mask;              /* bitwise-or of KM_OR_??? flags */
+    int64_t start, length;
+    char *cipher;
+    int64_t ivoffset;
+    void *km_data;
+} km_overrides_t;
+
+enum {
+    KM_OR_START =       0x001,
+    KM_OR_CIPHER =      0x002,
+    KM_OR_IVOFFSET =    0x004
+};
+
 
 const char **get_keymgr_list(void);
 keymanager_t *get_keymanager(const struct keyinfo *keyinfo, FILE *fp_key);
@@ -79,19 +107,27 @@ keymanager_t *get_keymanager(const struct keyinfo *keyinfo, FILE *fp_key);
 
 keymanager_t *kmblti_gethandle(void);
 keymanager_t *kmgcry_gethandle(void);
+keymanager_t *kmluks_gethandle(void);
 keymanager_t *init_keymanager(keymanager_t *km);
 int free_keymanagers(void);
 
-int is_keysafe(const struct keyinfo *keyinfo);
-int get_key(const char *ident, const struct keyinfo *keyinfo,
+unsigned cm_get_keyproperties(const struct keyinfo *keyinfo);
+int cm_get_key(const char *ident, const struct keyinfo *keyinfo,
             const struct km_pw_context *pw_ctxt,
-            unsigned char **key, int *keylen);
-int put_key(const char *ident, const struct keyinfo *keyinfo,
+            unsigned char **key, int *keylen,
+            km_overrides_t *overrides);
+int cm_put_key(const char *ident, const struct keyinfo *keyinfo,
             const struct km_pw_context *pw_ctxt,
-            const unsigned char *key, const int keylen, FILE *fp_key);
+            const unsigned char *key, const int keylen, FILE *fp_key,
+            const km_overrides_t *overrides);
 size_t mk_key_string(const unsigned char *key, const size_t keylen,
             char *buff);
 
+km_overrides_t *alloc_overrides();
+void merge_overrides(const struct cment *ent,
+            const km_overrides_t *overrides, struct cment *merged);
+void free_overrides(km_overrides_t *overrides);
+
 int sycheck_directory(const char *dirname);
 int sycheck_cmtab(const char *cmtab);
 int sycheck_target(const struct cment *ent);
@@ -115,6 +151,8 @@ int cm_mutex_unlock(void);
 
 #endif  /* _ARMOUR_H */
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
  *  (C)Copyright 2005-2008, RW Penney
  */
