@@ -1,6 +1,6 @@
 /*
  *  cryptmount - a utility for user-level mounting of encrypted filesystems
- *  $Revision: 213 $, $Date: 2008-07-26 07:28:58 +0100 (Sat, 26 Jul 2008) $
+ *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
  *  (C)Copyright 2005-2008, RW Penney
  */
 
@@ -217,7 +217,7 @@ static int64_t getblkcount(const char *device)
 
 
 static int do_list(const cment_t *cment)
-    /* print list of available filing-system targets */
+    /* Print list of available filing-system targets */
 {
     /* TRANSLATORS: this string is marked as 'no-c-format' because
        some localizations may require the mount-point and filesystem type
@@ -236,51 +236,30 @@ static int do_devsetup(const km_pw_context_t *pw_ctxt,
     /* setup all devices needed to access encrypted target */
 {   enum { BUFFMIN=1024 };
     unsigned char *key=NULL;
-    int i, killloop=0, keylen=0, eflag=ERR_NOERROR;
+    int i, readonly, isloop=0, killloop=0, keylen=0, eflag=ERR_NOERROR;
     int64_t devlen=0, fslen=0;
-    struct stat sbuff;
     size_t dpsize;
-    char *dmparams=NULL, *tgtdev=NULL, *loopdev=NULL;
+    char *dmparams=NULL;
+    const char *tgtdev=NULL;
+    km_overrides_t *overrides=NULL;
+    cment_t cment_aug;
 
     /* get crypto-key for filing system: */
-    eflag = get_key(cment->ident, &cment->key, pw_ctxt, &key, &keylen);
+    overrides = alloc_overrides();
+    eflag = cm_get_key(cment->ident, &cment->key, pw_ctxt,
+                        &key, &keylen, overrides);
     if (eflag != ERR_NOERROR) {
         fprintf(stderr, _("failed to extract cipher key\n"));
         goto bail_out;
     }
+    merge_overrides(cment, overrides, &cment_aug);
 
-    if (cment->dev == NULL || stat(cment->dev, &sbuff) != 0) {
-        fprintf(stderr, _("cannot stat device \"%s\" for target \"%s\"\n"),
-                (cment->dev != NULL ? cment->dev : "(NULL)"), cment->ident);
-        eflag = ERR_BADDEVICE;
-        goto bail_out;
-    }
-
-    /* check whether using raw device or loopback to file: */
-    if (S_ISBLK(sbuff.st_mode)) {
-        tgtdev = cment->dev;
-    } else if (S_ISREG(sbuff.st_mode)) {
-        if (cment->loopdev != NULL && strcmp(cment->loopdev, "auto") != 0) {
-            loopdev = (char*)malloc((size_t)(strlen(cment->loopdev) + 1));
-            strcpy(loopdev, cment->loopdev);
-        } else {
-            loopdev = (char*)malloc((size_t)BUFFMIN);
-            if (loop_findfree(loopdev, (size_t)BUFFMIN) != 0) {
-                fprintf(stderr, _("no available loopback devices\n"));
-                eflag = ERR_BADDEVICE;
-                goto bail_out;
-            }
-        }
-        eflag = loop_setup(loopdev, cment->dev,
-                        (is_readonlyfs(cment->dev) ? O_RDONLY : O_RDWR));
-        if (eflag != ERR_NOERROR) goto bail_out;
-        tgtdev = loopdev;
-    } else {
-        fprintf(stderr,
-            _("bad device type (%x) for \"%s\" (need block/file)\n"),
-            (unsigned)sbuff.st_mode, cment->dev);
-        tgtdev = NULL;
-        eflag = ERR_BADDEVICE;
+    readonly = is_readonlyfs(cment_aug.dev);
+    eflag = blockify_file(cment_aug.dev, (readonly ? O_RDONLY : O_RDWR),
+                        cment_aug.loopdev, &tgtdev, &isloop);
+    if (eflag != ERR_NOERROR) {
+        fprintf(stderr, _("cannot open device \"%s\" for target \"%s\"\n"),
+                (cment_aug.dev != NULL ? cment_aug.dev : "(NULL)"), cment_aug.ident);
         goto bail_out;
     }
 
@@ -291,15 +270,15 @@ static int do_devsetup(const km_pw_context_t *pw_ctxt,
         eflag = ERR_BADIOCTL;
         goto bail_out;
     }
-    if (cment->length < 0 || (cment->start + cment->length) > devlen) {
-        fslen = devlen - cment->start;
+    if (cment_aug.length < 0 || (cment_aug.start + cment_aug.length) > devlen) {
+        fslen = devlen - cment_aug.start;
     } else {
-        fslen = cment->length;
+        fslen = cment_aug.length;
     }
-    if (cment->start < 0 || fslen <= 0) {
+    if (cment_aug.start < 0 || fslen <= 0) {
         fprintf(stderr,_("bad device-mapper start/length"));
         fprintf(stderr, " (%" PRId64 ",%" PRId64 ")\n",
-                cment->start, cment->length);
+                cment_aug.start, cment_aug.length);
         eflag = ERR_BADDEVICE;
         goto bail_out;
     }
@@ -308,34 +287,32 @@ static int do_devsetup(const km_pw_context_t *pw_ctxt,
     dpsize = 2 * keylen + BUFFMIN;
     dmparams = sec_realloc(dmparams, dpsize);
     i = snprintf(dmparams, dpsize, "%s ",
-                (cment->cipher != NULL ? cment->cipher : DFLT_CIPHER));
+                (cment_aug.cipher != NULL ? cment_aug.cipher : DFLT_CIPHER));
     i += mk_key_string(key, (size_t)keylen, dmparams + i);
     snprintf(dmparams + i, (dpsize - i), " %" PRId64 " %s %" PRId64,
-                cment->ivoffset, tgtdev, cment->start);
+                cment_aug.ivoffset, tgtdev, cment_aug.start);
 
     /* setup device-mapper target: */
-    eflag = devmap_create(cment->ident,
+    eflag = devmap_create(cment_aug.ident,
             (uint64_t)0, (uint64_t)fslen, "crypt", dmparams);
     if (eflag != ERR_NOERROR) {
         fprintf(stderr,
             _("device-mapper target-creation failed for \"%s\"\n"),
-            cment->ident);
+            cment_aug.ident);
         killloop = 1;
         goto bail_out;
     }
     if (mntdev != NULL) {
-        devmap_path(mntdev, cment->ident);
+        devmap_path(mntdev, cment_aug.ident);
     }
 
 
   bail_out:
 
-    if (loopdev != NULL) {
-        if (killloop) loop_destroy(loopdev);    /* mounting failed? */
-        free((void*)loopdev);
-    }
+    if (killloop) unblockify_file(&tgtdev, isloop);   /* mounting failed? */
     sec_free(dmparams);
     sec_free(key);
+    free_overrides(overrides);
 
     return eflag;
 }   /* do_devsetup() */
@@ -571,44 +548,55 @@ static int do_swapoff(const cment_t *cment)
 
 
 static int do_passwd(const km_pw_context_t *pw_ctxt, const cment_t *cment)
+    /* Change password on particular target */
 {   unsigned char *key=NULL;
+    unsigned keyprops;
     int keylen=0,eflag=ERR_NOERROR;
-    char *newfname=NULL,*oldfname=NULL;
+    char *newfname=NULL, *oldfname=NULL;
     struct stat sbuff;
     size_t sz;
-    FILE *fp;
+    FILE *fp=NULL;
+    km_overrides_t *overrides=NULL;
 
-    if (!is_keysafe(&cment->key)) {
-        fprintf(stderr, _("key-file for \"%s\" isn't password-protected\n"),
+    keyprops = cm_get_keyproperties(&cment->key);
+    if ((keyprops & KM_PROP_HASPASSWD) == 0) {
+        fprintf(stderr, _("Key-file for \"%s\" isn't password-protected\n"),
                 cment->ident);
         eflag = WRN_NOPASSWD;
         goto bail_out;
     }
 
-    eflag = get_key(cment->ident, &cment->key, pw_ctxt, &key, &keylen);
+    overrides = alloc_overrides();
+    eflag = cm_get_key(cment->ident, &cment->key, pw_ctxt,
+                        &key, &keylen, overrides);
     if (eflag != ERR_NOERROR) goto bail_out;
 
-    sz = strlen(cment->key.filename) + 16;
-    oldfname = (char*)malloc(2 * sz);
-    newfname = oldfname + sz;
-    snprintf(oldfname, sz, "%s-old", cment->key.filename);
-    snprintf(newfname, sz, "%s-new", cment->key.filename);
-
-    fp = fopen(newfname, "wb");
+    if ((keyprops & KM_PROP_FIXEDLOC) == 0) {
+        sz = strlen(cment->key.filename) + 16;
+        oldfname = (char*)malloc(2 * sz);
+        newfname = oldfname + sz;
+        snprintf(oldfname, sz, "%s-old", cment->key.filename);
+        snprintf(newfname, sz, "%s-new", cment->key.filename);
+        fp = fopen(newfname, "wb");
+    } else {
+        fp = fopen(cment->key.filename, "r+b");
+    }
     if (fp == NULL) {
-        fprintf(stderr, _("cannot open \"%s\" for writing\n"), newfname);
+        fprintf(stderr, _("Cannot open \"%s\" for writing\n"), newfname);
         eflag = ERR_BADFILE;
         goto bail_out;
     }
-    eflag = put_key(cment->ident, &cment->key, pw_ctxt, key, keylen, fp);
+    eflag = cm_put_key(cment->ident, &cment->key, pw_ctxt,
+                        key, keylen, fp, overrides);
     if (fclose(fp) != 0) eflag = ERR_BADFILE;
+    if (eflag != ERR_NOERROR) goto bail_out;
 
-    if (eflag == ERR_NOERROR) {
+    if ((keyprops & KM_PROP_FIXEDLOC) == 0) {
         if (stat(cment->key.filename, &sbuff) != 0
           || rename(cment->key.filename, oldfname) != 0
           || chown(oldfname, 0, 0) != 0
           || chmod(oldfname, S_IRUSR | S_IWUSR) != 0) {
-            fprintf(stderr, _("retiring old key (%s -> %s) failed\n"),
+            fprintf(stderr, _("Retiring old key (%s -> %s) failed\n"),
                     cment->key.filename, oldfname);
             goto bail_out;
         }
@@ -616,14 +604,14 @@ static int do_passwd(const km_pw_context_t *pw_ctxt, const cment_t *cment)
         if (rename(newfname, cment->key.filename) != 0
           || chown(cment->key.filename, sbuff.st_uid, sbuff.st_gid) != 0
           || chmod(cment->key.filename, sbuff.st_mode) != 0) {
-            fprintf(stderr, _("installing new key (%s -> %s) failed\n"),
+            fprintf(stderr, _("Installing new key (%s -> %s) failed\n"),
                     newfname, cment->key.filename);
             goto bail_out;
         }
 
         newfname = NULL;
 
-        fprintf(stderr, _("backup of previous key is in \"%s\"\n"), oldfname);
+        fprintf(stderr, _("Backup of previous key is in \"%s\"\n"), oldfname);
     }
 
   bail_out:
@@ -632,6 +620,7 @@ static int do_passwd(const km_pw_context_t *pw_ctxt, const cment_t *cment)
         unlink(newfname);
     }
     if (oldfname != NULL) free((void*)oldfname);
+    free_overrides(overrides);
 
     return eflag;
 }
@@ -639,70 +628,81 @@ static int do_passwd(const km_pw_context_t *pw_ctxt, const cment_t *cment)
 
 static int do_keygen(const km_pw_context_t *pw_ctxt, const cment_t *cment,
                     const char *params, int reuse, const cment_t *cmtable)
-    /* create new filesystem crypto-key */
+    /* Create new filesystem crypto-key */
 {   unsigned char *key=NULL;
-    int keylen=0, eflag=ERR_NOERROR;
+    unsigned keyprops, propmask;
+    int keylen=0, fileexists=0, eflag=ERR_NOERROR;
     char *fname=NULL;
     const cment_t *parent=NULL;
     size_t sz;
     struct stat sbuff;
-    FILE *fp;
+    FILE *fp=NULL;
 
     if (params != NULL) {
         if (reuse) {
             parent = get_cment(cmtable, params);
             if (parent == NULL) {
-                fprintf(stderr, _("target name \"%s\" is not recognized\n"),
+                fprintf(stderr, _("Target name \"%s\" is not recognized\n"),
                         params);
                 eflag = ERR_BADPARAM;
             }
         } else {
             if (sscanf(params, "%i", &keylen) != 1 || keylen < 1) {
-                fprintf(stderr, _("bad key-length parameter"));
+                fprintf(stderr, _("Bad key-length parameter"));
                 eflag = ERR_BADPARAM;
             }
         }
     }
     if (params == NULL || eflag != ERR_NOERROR) goto bail_out;
 
-    /* check if keyfile already exists: */
-    if (stat(cment->key.filename, &sbuff) == 0) {
-        fprintf(stderr,_("key-file \"%s\" already exists for target \"%s\"\n"),
+    /* Check if keyfile already exists: */
+    keyprops = cm_get_keyproperties(&cment->key);
+    propmask = KM_PROP_FIXEDLOC | KM_PROP_FORMATTED;
+    fileexists = (stat(cment->key.filename, &sbuff) == 0);
+    if (fileexists && (keyprops & propmask) != KM_PROP_FIXEDLOC) {
+        fprintf(stderr,_("Key-file \"%s\" already exists for target \"%s\"\n"),
             cment->key.filename, cment->ident);
         eflag = ERR_BADFILE;
         goto bail_out;
     }
 
     if (reuse) {
-        eflag = get_key(parent->ident, &parent->key, pw_ctxt, &key, &keylen);
+        eflag = cm_get_key(parent->ident, &parent->key, pw_ctxt,
+                        &key, &keylen, NULL);
     } else {
-        fprintf(stderr, _("generating random key; please be patient...\n"));
+        fprintf(stderr, _("Generating random key; please be patient...\n"));
         key = (unsigned char*)sec_realloc(NULL, (size_t)keylen);
         eflag = get_randkey(key, (size_t)keylen);
         if (eflag != ERR_NOERROR) {
-            fprintf(stderr, _("failed to generate new key\n"));
+            fprintf(stderr, _("Failed to generate new key\n"));
             goto bail_out;
         }
     }
 
-    sz = strlen(cment->key.filename) + 16;
-    fname = (char*)malloc(sz);
-    snprintf(fname, sz, "%s-new", cment->key.filename);
+    if ((keyprops & KM_PROP_FIXEDLOC) == 0) {
+        sz = strlen(cment->key.filename) + 16;
+        fname = (char*)malloc(sz);
+        snprintf(fname, sz, "%s-new", cment->key.filename);
+        fp = fopen(fname, "wb");
+    } else {
+        fp = fopen(cment->key.filename, (fileexists ? "r+b" : "wb"));
+    }
 
-    fp = fopen(fname, "wb");
     if (fp == NULL) {
-        fprintf(stderr, _("cannot open \"%s\" for writing\n"), fname);
+        fprintf(stderr, _("Cannot open \"%s\" for writing\n"), fname);
         eflag = ERR_BADFILE;
         goto bail_out;
     }
-    eflag = put_key(cment->ident, &cment->key, pw_ctxt, key, keylen, fp);
-    fclose(fp);
+    eflag = cm_put_key(cment->ident, &cment->key, pw_ctxt,
+                        key, keylen, fp, NULL);
+    if (fclose(fp) != 0) eflag = ERR_BADFILE;
+    if (eflag != ERR_NOERROR) goto bail_out;
 
-    if (eflag == ERR_NOERROR) {
+    if ((keyprops & KM_PROP_FIXEDLOC) == 0) {
         if (rename(fname, cment->key.filename) != 0
           || chown(cment->key.filename, 0, 0) != 0
           || chmod(cment->key.filename, S_IRUSR | S_IWUSR) != 0) {
-            fprintf(stderr, _("installation of new keyfile \"%s\" failed"),
+            fprintf(stderr, _("Installation of new keyfile \"%s\" failed"),
                     cment->key.filename);
             eflag = ERR_BADFILE;
         }
@@ -718,7 +718,7 @@ static int do_keygen(const km_pw_context_t *pw_ctxt, const cment_t *cment,
 
 
 static int do_safetynet()
-    /* last-ditch attempt to umount/shutdown all targets currently mounted */
+    /* Last-ditch attempt to umount/shutdown all targets currently mounted */
 {   tgtstat_t *tgts=NULL, *tgt;
     char *devname=NULL, *old_ident=NULL;
     cment_t *cment=NULL;
@@ -1243,7 +1243,8 @@ int main(int argc, char *argv[])
     return eflag;
 }
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
- *  cryptmount.c
  *  (C)Copyright 2005-2008, RW Penney
  */
