@@ -1,6 +1,6 @@
 /*
  *  general declarations for cryptmount
- *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
  *  (C)Copyright 2005-2008, RW Penney
  */
 
@@ -95,7 +95,8 @@ enum    /* error flags */
     ERR_INSECURE,
     ERR_BADPASSWD,
     ERR_BADPARAM,
-    ERR_BADMUTEX
+    ERR_BADMUTEX,
+    ERR_ABORT
 };
 
 
