@@ -1,6 +1,6 @@
 /*
  *  filesystem-related utilities for cryptmount
- *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
  *  (C)Copyright 2005-2008, RW Penney
  */
 
@@ -61,7 +61,7 @@ int do_rmvmntent(const cment_t *cment);
 #if ERSATZ_MOUNT
 
 int fs_mount(const char *mntdev, const cment_t *cment)
-    /* fallback version of mount(8) using mount(2) */
+    /* Fallback version of mount(8) using mount(2) */
 {   unsigned long mflags;
     int eflag=ERR_NOERROR;
     char *errstr=NULL;
@@ -88,11 +88,11 @@ int fs_mount(const char *mntdev, const cment_t *cment)
 #else   /* !ERSATZ_MOUNT */
 
 int fs_mount(const char *dev, const cment_t *cment)
-    /* delegate filesystem mounting to mount(8) */
+    /* Delegate filesystem mounting to mount(8) */
 {   int idx,eflag=ERR_NOERROR;
     const char *argv[16];
 
-    /* construct argument list for mount -t ... -o ... <dev> <dir> */
+    /* Construct argument list for mount -t ... -o ... <dev> <dir> */
     idx = 0;
     argv[idx++] = "[cryptmount-mount]";
     if (cment->fstype != NULL) {
@@ -115,7 +115,7 @@ int fs_mount(const char *dev, const cment_t *cment)
 #if ERSATZ_UMOUNT
 
 int fs_unmount(const cment_t *cment)
-    /* fallback version of umount(8) using umount(2) */
+    /* Fallback version of umount(8) using umount(2) */
 {   int eflag=ERR_NOERROR;
     char *errstr=NULL;
 
@@ -137,11 +137,11 @@ int fs_unmount(const cment_t *cment)
 #else   /* !ERSATZ_UMOUNT */
 
 int fs_unmount(const cment_t *cment)
-    /* delegate filesystem mounting to umount(8) */
+    /* Delegate filesystem mounting to umount(8) */
 {   int idx,eflag=ERR_NOERROR;
     const char *argv[16];
 
-    /* construct argument list for umount -t ... <dir> */
+    /* Construct argument list for umount -t ... <dir> */
     idx = 0;
     argv[idx++] = "[cryptmount-umount]";
 #ifdef PARANOID
@@ -233,7 +233,7 @@ int fs_swapoff(const cment_t *cment)
 
 
 int parse_fsoptions(const char *buff, unsigned long *mflags)
-    /* convert string of mount-options into binary flags */
+    /* Convert string of mount-options into binary flags */
 {   struct fsopt_t {
         const char *str;
         unsigned long mask; };
@@ -273,7 +273,7 @@ int parse_fsoptions(const char *buff, unsigned long *mflags)
 
 
 int do_addmntent(const cment_t *cment)
-    /* add entry into /etc/mtab for newly-mounted filing system */
+    /* Add entry into /etc/mtab for newly-mounted filing system */
 {   char *mntdev=NULL;
     struct mntent mntinfo;
     FILE *fp;
@@ -305,7 +305,7 @@ int do_addmntent(const cment_t *cment)
 
 
 int do_rmvmntent(const cment_t *cment)
-    /* remove entry from /etc/mtab after unmounting filing system */
+    /* Remove entry from /etc/mtab after unmounting filing system */
 {   char *mntdev=NULL;
     struct mntent *mntinfo;
     FILE *fp_in=NULL,*fp_out=NULL;
@@ -316,7 +316,7 @@ int do_rmvmntent(const cment_t *cment)
 
     devmap_path(&mntdev, cment->ident);
 
-    /* open old /etc/mtab & create temporary replacement: */
+    /* Open old /etc/mtab & create temporary replacement: */
     fp_in = setmntent(ETCMTAB, "r");
     fp_out = setmntent(ETCMTABTMP, "w");
     if (fp_in == NULL || fp_out == NULL) {
@@ -331,7 +331,7 @@ int do_rmvmntent(const cment_t *cment)
         goto bail_out;
     }
 
-    /* transfer entries from old /etc/mtab to prototype replacement file: */
+    /* Transfer entries from old /etc/mtab to prototype replacement file: */
     while ((mntinfo = getmntent(fp_in)) != NULL) {
         if (strcmp(mntinfo->mnt_fsname, mntdev) == 0
           && strcmp(mntinfo->mnt_dir, cment->dir) == 0) {
@@ -341,7 +341,7 @@ int do_rmvmntent(const cment_t *cment)
         }
     }
 
-    /* transfer ownership & permissions from old /etc/mtab to new: */
+    /* Transfer ownership & permissions from old /etc/mtab to new: */
     if (chown(ETCMTABTMP, sbuff.st_uid, sbuff.st_gid) != 0
       || chmod(ETCMTABTMP, sbuff.st_mode) != 0) {
         fprintf(stderr, "cannot transfer ownership/modes to \"%s\"\n",
@@ -353,7 +353,7 @@ int do_rmvmntent(const cment_t *cment)
     endmntent(fp_in); fp_in = NULL;
 
     if (rename(ETCMTABTMP, ETCMTAB)) {
-        fprintf(stderr, "failed to recreate %s\n", ETCMTAB);
+        fprintf(stderr, "Failed to recreate %s\n", ETCMTAB);
     }
 
   bail_out:
@@ -367,7 +367,7 @@ int do_rmvmntent(const cment_t *cment)
 
 
 int is_mounted(const cment_t *cment)
-    /* check if target is mounted */
+    /* Check if target is mounted */
 {   int mounted=0;
     char *mntdev=NULL;
     struct mntent *mntinfo;
@@ -405,7 +405,9 @@ int is_readonlyfs(const char *path)
     /* check if filesystem containing *path is read-only */
 {   struct statvfs sbuff;
 
-    return ((statvfs(path, &sbuff) != 0 || (sbuff.f_flag & ST_RDONLY) != 0));
+    return (path == NULL
+            || (statvfs(path, &sbuff) != 0
+            || (sbuff.f_flag & ST_RDONLY) != 0));
 }
 
 
