@@ -1,6 +1,6 @@
 /*
  *  loopback-device utilities for cryptmount
- *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
  *  (C)Copyright 2005-2008, RW Penney
  */
 
@@ -78,7 +78,7 @@ int loop_findfree(char *buff, size_t buffsz)
     str[0] = '\0';
     for (min=0; min<256u && !found; ++min) {
         for (idx=0; loop_formats[idx]!=NULL && !found; ++idx) {
-            sprintf(str, loop_formats[idx], min);
+            snprintf(str, sizeof(str), loop_formats[idx], min);
             if (stat(str, &sbuff) || !S_ISBLK(sbuff.st_mode)) continue;
             devfd = open(str, O_RDONLY);
             if (devfd < 0) continue;
@@ -205,6 +205,75 @@ int loop_dellist(unsigned devcnt, const dev_t *devids)
     return eflag;
 }
 
+
+int blockify_file(const char *filename, int fmode, const char *prefdev,
+                const char **devname, int *isloop)
+    /* convert <filename> to block-device if it isn't already */
+{   enum { BUFFMIN=1024 };
+    struct stat sbuff;
+    char *loopdev=NULL;
+    int eflag=ERR_NOERROR;
+
+    if (filename == NULL || stat(filename, &sbuff) != 0) {
+        *isloop = 0;
+        eflag = ERR_BADDEVICE;
+        goto bail_out;
+    }
+
+    if (S_ISBLK(sbuff.st_mode)) {
+        /* keyfile is block-special already: */
+        *devname = filename;
+        *isloop = 0;
+    } else if (S_ISREG(sbuff.st_mode)) {
+        /* create loopback device around ordinary file: */
+        if (prefdev != NULL && strcmp(prefdev, "auto") != 0) {
+            loopdev = (char*)malloc((size_t)(strlen(prefdev) + 1));
+            strcpy(loopdev, prefdev);
+        } else {
+            loopdev = (char*)malloc((size_t)BUFFMIN);
+            if (loop_findfree(loopdev, (size_t)BUFFMIN) != 0) {
+                fprintf(stderr, _("no available loopback devices\n"));
+                eflag = ERR_BADDEVICE;
+                goto bail_out;
+            }
+        }
+        if (loop_setup(loopdev, filename, fmode) != ERR_NOERROR) {
+            eflag = ERR_BADDEVICE;
+            goto bail_out;
+        }
+        *devname = loopdev;
+        loopdev = NULL;
+        *isloop = 1;
+    } else {
+        fprintf(stderr,
+            _("bad device type (%x) for \"%s\" (need block/file)\n"),
+            (unsigned)sbuff.st_mode, filename);
+        *devname = NULL;
+        *isloop = 0;
+        eflag = ERR_BADDEVICE;
+    }
+
+  bail_out:
+
+    if (loopdev != NULL) free((void*)loopdev);
+
+    return eflag;
+}
+
+
+int unblockify_file(const char **devname, int isloop)
+    /* remove loopback device previously created by blockify_file() */
+{   int eflag = ERR_NOERROR;
+
+    if (isloop && *devname != NULL) {
+        eflag = loop_destroy(*devname);
+        free((void*)*devname);
+        *devname = NULL;
+    }
+
+    return eflag;
+}
+
 /*
  *  (C)Copyright 2005-2008, RW Penney
  */
