@@ -1,6 +1,6 @@
 /*
  *  declarations for loopback-device utilities for cryptmount
- *  $Revision: 202 $, $Date: 2008-06-07 10:29:31 +0100 (Sat, 07 Jun 2008) $
+ *  $Revision: 222 $, $Date: 2008-10-03 12:31:16 +0100 (Fri, 03 Oct 2008) $
  *  (C)Copyright 2005-2008, RW Penney
  */
 
@@ -34,6 +34,10 @@ int loop_ident(unsigned maj, unsigned min, char *buff, size_t buffsz);
 int loop_destroy(const char *dev);
 int loop_dellist(unsigned devcnt, const dev_t *devids);
 
+int blockify_file(const char *filename, int fmode, const char *prefdev,
+                const char **devname, int *isloop);
+int unblockify_file(const char **devname, int isloop);
+
 
 #endif  /* _LOOPUTILS_H */
 
