@@ -46,7 +46,7 @@
 
 
 int cm_path(char **buff, const char *file)
-    /* create full pathname of config file */
+    /* Create full pathname of config file */
 {   size_t pfxlen, sfxlen;
     const char *pfx=NULL;
 
@@ -69,7 +69,7 @@ int cm_path(char **buff, const char *file)
 
 
 char *cm_strdup(const char *orig)
-    /* make duplicate of existing string, reserving memory */
+    /* Make duplicate of existing string, reserving memory */
 {   char *cpy;
 
     if (orig == NULL) return NULL;
@@ -90,30 +90,30 @@ int cm_strcasecmp(const char *s1, const char *s2)
 }
 
 void *sec_realloc(void *ptr, size_t size)
-    /* slightly more secure version of realloc() */
+    /* Slightly more secure version of realloc() */
 {   size_t cnt, *memarr;
 
     cnt = (size + 2 * sizeof(size_t) - 1) / sizeof(size_t);
     memarr = (size_t*)calloc(cnt, sizeof(size_t));
 
     if (memarr == NULL) {
-        fprintf(stderr, _("unable to allocated memory\n"));
+        fprintf(stderr, _("Unable to allocate memory\n"));
         abort();
         return NULL;
     }
 
-    /* prepend usable memory chunk with record of size of chunk: */
+    /* Prepend usable memory chunk with record of size of chunk: */
     memarr[0] = (cnt - 1) * sizeof(size_t);
 
     if (ptr != NULL) {
         size_t oldsz;
 
-        /* copy (usable) part of old memory block into new: */
+        /* Copy (usable) part of old memory block into new: */
         oldsz = *(((size_t*)ptr) - 1);
         if (oldsz > size) oldsz = size;
         memcpy((void*)(memarr + 1), (const void*)ptr, size);
 
-        /* dispose of old memory block: */
+        /* Dispose of old memory block: */
         sec_free(ptr);
     }
 
@@ -122,7 +122,7 @@ void *sec_realloc(void *ptr, size_t size)
 
 
 void mem_cleanse(unsigned char *addr, size_t sz)
-    /* overwrite memory with (weak) pseudo-random numbers */
+    /* Overwrite memory with (weak) pseudo-random numbers */
 {   size_t i;
     static unsigned long salt=0x917c;
 
@@ -136,7 +136,7 @@ void mem_cleanse(unsigned char *addr, size_t sz)
 
 
 void sec_free(void *ptr)
-    /* slightly more secure version of free() */
+    /* Slightly more secure version of free() */
 {   size_t *memarr, sz;
 
     if (ptr == NULL) return;
@@ -151,13 +151,13 @@ void sec_free(void *ptr)
 
 
 int get_randkey(unsigned char *buff, size_t len)
-    /* generate random string of bytes */
+    /* Generate random string of bytes */
 {   struct rnddev {
         const char *name;
         unsigned devmaj, devmin;
         FILE *fp; };
     struct rnddev devs[] = {
-#if 0 || !defined(TESTING)
+#if !defined(TESTING)
         { "/dev/hwrng",     MISC_MAJOR, 183,    NULL },
         { "/dev/random",    MEM_MAJOR, 8,       NULL },
 #endif
@@ -182,7 +182,7 @@ int get_randkey(unsigned char *buff, size_t len)
         if ((devs[i].fp = fopen(devs[i].name,"rb")) != NULL) ++cnt;
     }
     if (cnt == 0) {
-        fprintf(stderr, _("no random-number devices found\n"));
+        fprintf(stderr, _("No random-number devices found\n"));
         eflag = WRN_LOWENTROPY;
     }
     blklen = (len > CM_SHA1_SIZE ? (size_t)CM_SHA1_SIZE : len);
@@ -191,7 +191,7 @@ int get_randkey(unsigned char *buff, size_t len)
 
     pid = getpid();
 
-    /*  combine multiple sources of entropy
+    /*  Combine multiple sources of entropy
         (should still be effective if subset aren't viable): */
     pos = 0;
     while (pos < len) {
@@ -238,7 +238,7 @@ int get_randkey(unsigned char *buff, size_t len)
 
 
 ssize_t cm_ttygetpasswd(const char *prompt, char **buff)
-    /* read password from standard input terminal */
+    /* Read password from standard input terminal */
 {   ssize_t pwlen=0;
 #if HAVE_TERMIOS
     struct termios oldttystate, newttystate;
@@ -257,7 +257,10 @@ ssize_t cm_ttygetpasswd(const char *prompt, char **buff)
       || (newttystate.c_lflag & ECHO) != 0) echook = 0;
     if (echook) {
         printf(prompt);
-        fgets(tmppass, (int)sizeof(tmppass), stdin);
+        if (fgets(tmppass, (int)sizeof(tmppass), stdin) == NULL) {
+            fprintf(stderr, _("Cannot read stdin"));
+            return -1;
+        }
         pwlen = strlen(tmppass);
         if (pwlen > 0 && tmppass[pwlen-1] == '\n') {
             tmppass[--pwlen] = '\0'; }
@@ -267,7 +270,7 @@ ssize_t cm_ttygetpasswd(const char *prompt, char **buff)
         tcsetattr(fileno(stdin), TCSAFLUSH, &oldttystate);
         printf("\n");
     } else {
-        fprintf(stderr, _("failed to turn off keyboard echoing on terminal\n"));
+        fprintf(stderr, _("Failed to turn off keyboard echoing on terminal\n"));
         pwlen = -1;
     }
 #else
@@ -284,7 +287,7 @@ ssize_t cm_ttygetpasswd(const char *prompt, char **buff)
 
 int km_get_passwd(const char *ident, const km_pw_context_t *pw_ctxt,
                 char **passwd, int isnew, int verify)
-    /* read password from terminal, possibly asking for confirmation */
+    /* Read password from terminal, possibly asking for confirmation */
 {   enum { BUFFSZ=2048 };
     char *tmppass=NULL;
     ssize_t plen=0;
@@ -292,11 +295,11 @@ int km_get_passwd(const char *ident, const km_pw_context_t *pw_ctxt,
 
     if (pw_ctxt == NULL || pw_ctxt->fd_pw_source == NULL) {
 #ifndef TESTING
-        /* read (+confirm) password from terminal: */
+        /* Read (+confirm) password from terminal: */
 	char prompt[BUFFSZ];
         snprintf(prompt, sizeof(prompt),
-                (isnew ? _("enter new password for target \"%s\": ")
-                    : _("enter password for target \"%s\": ")),
+                (isnew ? _("Enter new password for target \"%s\": ")
+                    : _("Enter password for target \"%s\": ")),
                 ident);
 
         if (cm_ttygetpasswd(prompt, passwd) < 0) {
@@ -305,28 +308,31 @@ int km_get_passwd(const char *ident, const km_pw_context_t *pw_ctxt,
         }
 
         if (verify) {
-            snprintf(prompt, sizeof(prompt), _("confirm password: "));
+            snprintf(prompt, sizeof(prompt), _("Confirm password: "));
             plen = cm_ttygetpasswd(prompt, &tmppass);
             if (strcmp(*passwd, tmppass) != 0) {
-                fprintf(stderr, _("password mismatch\n"));
+                fprintf(stderr, _("Password mismatch\n"));
                 sec_free(*passwd);
                 *passwd = NULL;
                 eflag = ERR_BADPASSWD;
             }
         }
 #else   /* TESTING */
-        /* read passwords passed in via command-line arguments: */
+        /* Read passwords passed in via command-line arguments: */
         const char *argpw;
         argpw = (pw_ctxt != NULL ? pw_ctxt->argpasswd[(isnew ? 1 : 0)] : NULL);
         *passwd = (char*)sec_realloc((void*)*passwd, (size_t)1024);
         strncpy(*passwd, (argpw != NULL ? argpw : ""), (size_t)1024);
 #endif
     } else {
-        /* read password (once only) from input stream: */
+        /* Read password (once only) from input stream: */
         tmppass = (char*)sec_realloc(tmppass, (size_t)BUFFSZ);
-        fgets(tmppass, BUFFSZ, pw_ctxt->fd_pw_source);
+        if (fgets(tmppass, BUFFSZ, pw_ctxt->fd_pw_source) == NULL) {
+            eflag = ERR_BADFILE;
+            goto bail_out;
+        }
 
-        /* remove trailing carriage-return(s): */
+        /* Remove trailing carriage-return(s): */
         plen = strlen(tmppass);
         while (plen > 0 && tmppass[plen-1] == '\n') tmppass[--plen] = '\0';
 
@@ -342,8 +348,30 @@ int km_get_passwd(const char *ident, const km_pw_context_t *pw_ctxt,
 }
 
 
+int cm_confirm(const char *msg)
+    /* Invite user to pause before taking dangerous action */
+{   char *affirmativeResponse=_("yes");
+    char response[64];
+    int rlen;
+
+    if (msg != NULL) {
+        printf("%s\n", msg);
+    }
+    fprintf(stdout, _("Are you sure? (Type \"%s\" to proceed): "), affirmativeResponse);
+
+    if (fgets(response, (int)sizeof(response), stdin) == NULL) {
+        fprintf(stderr, _("Cannot read stdin\n"));
+        return 0;
+    }
+    rlen = strlen(response);
+    if (rlen > 0 && response[rlen-1] == '\n') response[--rlen] = '\0';
+
+    return (cm_strcasecmp(response, affirmativeResponse) == 0);
+}
+
+
 unsigned km_aug_keysz(unsigned keylen, unsigned blksz)
-    /* calculate size of augmented cipher-key after appending checksum etc */
+    /* Calculate size of augmented cipher-key after appending checksum etc */
 {
     return blksz * ((keylen + 2 * sizeof(uint32_t) + blksz - 1) / blksz);
 }
@@ -351,7 +379,7 @@ unsigned km_aug_keysz(unsigned keylen, unsigned blksz)
 
 unsigned char *km_aug_key(const unsigned char *key, unsigned keylen,
                     unsigned blocksz, size_t *buffsz)
-    /* augment cipher key with checksum prior to encryption & storage */
+    /* Augment cipher key with checksum prior to encryption & storage */
 {   unsigned char *buff=NULL;
     uint32_t chksum, *kptr=NULL;
     size_t idx, cnt;
@@ -359,11 +387,11 @@ unsigned char *km_aug_key(const unsigned char *key, unsigned keylen,
     *buffsz = km_aug_keysz(keylen, blocksz);
     buff = (unsigned char*)sec_realloc(buff, *buffsz);
 
-    /* copy key into zero-padded buffer: */
+    /* Copy key into zero-padded buffer: */
     memset(buff, 0, (size_t)*buffsz);
     memcpy(buff, key, (size_t)keylen);
 
-    /* compute crude EOR checksum (invariant to byte-ordering): */
+    /* Compute crude EOR checksum (invariant to byte-ordering): */
     cnt = (keylen + sizeof(chksum) - 1) / sizeof(chksum);
     chksum = 0;
     kptr = (uint32_t*)buff;
@@ -372,7 +400,7 @@ unsigned char *km_aug_key(const unsigned char *key, unsigned keylen,
         ++kptr;
     }
 
-    /* install checksum at next 4-byte boundary & pad with noise: */
+    /* Install checksum at next 4-byte boundary & pad with noise: */
     *kptr = chksum;
     idx = (idx + 1) * sizeof(chksum);
     if (idx < *buffsz) {
@@ -385,7 +413,7 @@ unsigned char *km_aug_key(const unsigned char *key, unsigned keylen,
 
 int km_aug_verify(const unsigned char *buff, unsigned keylen,
                 uint32_t *expected, uint32_t *actual)
-    /* check augmented cipher key against simple checksum */
+    /* Check augmented cipher key against simple checksum */
 {   unsigned cnt;
     uint32_t *kptr;
 
@@ -444,7 +472,7 @@ void cm_sha1_block(cm_sha1_ctxt_t *ctxt, const unsigned char *buff, size_t len)
     unsigned idx, round;
 
     while (len > 0) {
-        /* accumulate bytes into buffer (respecting endianess): */
+        /* Accumulate bytes into buffer (respecting endianess): */
         idx = ctxt->buffpos >> 2;
         round = 3 - (ctxt->buffpos & 0x03);
         ctxt->buff[idx] |= ((uint32_t)*buff) << (round * 8);
@@ -454,7 +482,7 @@ void cm_sha1_block(cm_sha1_ctxt_t *ctxt, const unsigned char *buff, size_t len)
         --len;
 
         if (ctxt->buffpos >= 64) {
-            /* whole 512-bit string is ready - apply SHA1 update to block: */
+            /* Whole 512-bit string is ready - apply SHA1 update to block: */
             for (idx=0; idx<16; ++idx) W[idx] = ctxt->buff[idx];
             for (idx=16; idx<80; ++idx) {
                 q = W[idx-3] ^ W[idx-8] ^ W[idx-14] ^ W[idx-16];
@@ -508,7 +536,7 @@ void cm_sha1_final(cm_sha1_ctxt_t *ctxt, unsigned char **mdval, size_t *mdlen)
     unsigned idx, padlen;
     uint32_t msglen;
 
-    /* add closing sequence onto message string: */
+    /* Add closing sequence onto message string: */
     msglen = ctxt->msglen;
     for (idx=0; idx<64; ++idx) buff[idx] = 0;
     padlen = (ctxt->buffpos < 56 ? 55 - ctxt->buffpos : 119 - ctxt->buffpos);
@@ -520,7 +548,7 @@ void cm_sha1_final(cm_sha1_ctxt_t *ctxt, unsigned char **mdval, size_t *mdlen)
     buff[7] = msglen & 0xff;
     cm_sha1_block(ctxt, buff, (size_t)8);
 
-    /* transcribe internal state into array of bytes: */
+    /* Transcribe internal state into array of bytes: */
     *mdval = (unsigned char*)sec_realloc(NULL, (size_t)CM_SHA1_SIZE);
     *mdlen = CM_SHA1_SIZE;
     cptr = *mdval;
@@ -543,7 +571,7 @@ void cm_sha1_free(cm_sha1_ctxt_t *ctxt)
 #ifdef TESTING
 
 int ut_test_sha1()
-    /* check internal SHA1 hashing algorithm against known test-vectors */
+    /* Check internal SHA1 hashing algorithm against known test-vectors */
 {   cm_sha1_ctxt_t *ctxt;
     unsigned char *mdval;
     unsigned b, q, idx;
@@ -582,7 +610,8 @@ int ut_test_sha1()
 
 #endif  /* TESTING */
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
- *  utils.c
  *  (C)Copyright 2005-2008, RW Penney
  */
