@@ -49,6 +49,7 @@ void sec_free(void *ptr);
 int get_randkey(unsigned char *key, size_t len);
 int km_get_passwd(const char *ident, const km_pw_context_t *pw_ctxt,
                 char **passwd, int isnew, int verify);
+int cm_confirm(const char *msg);
 unsigned km_aug_keysz(unsigned keylen, unsigned blksz);
 unsigned char *km_aug_key(const unsigned char *key, unsigned keylen,
                 unsigned blocksz, size_t *buffsz);
@@ -70,7 +71,8 @@ void cm_sha1_free(cm_sha1_ctxt_t *ctxt);
 
 #endif  /* _UTILS_H */
 
+/* vim: set ts=4 sw=4 et: */
+
 /*
- *  utils.h
  *  (C)Copyright 2005-2008, RW Penney
  */
